@@ -27,9 +27,15 @@ from ...utils import TensorType, is_torch_available, logging
 logger = logging.get_logger(__name__)
 
 BIGBIRD_PEGASUS_PRETRAINED_CONFIG_ARCHIVE_MAP = {
-    "google/bigbird-pegasus-large-arxiv": "https://huggingface.co/google/bigbird-pegasus-large-arxiv/resolve/main/config.json",
-    "google/bigbird-pegasus-large-pubmed": "https://huggingface.co/google/bigbird-pegasus-large-pubmed/resolve/main/config.json",
-    "google/bigbird-pegasus-large-bigpatent": "https://huggingface.co/google/bigbird-pegasus-large-bigpatent/resolve/main/config.json",
+    "google/bigbird-pegasus-large-arxiv": (
+        "https://huggingface.co/google/bigbird-pegasus-large-arxiv/resolve/main/config.json"
+    ),
+    "google/bigbird-pegasus-large-pubmed": (
+        "https://huggingface.co/google/bigbird-pegasus-large-pubmed/resolve/main/config.json"
+    ),
+    "google/bigbird-pegasus-large-bigpatent": (
+        "https://huggingface.co/google/bigbird-pegasus-large-bigpatent/resolve/main/config.json"
+    ),
     # See all BigBirdPegasus models at https://huggingface.co/models?filter=bigbird_pegasus
 }
 
@@ -79,10 +85,10 @@ class BigBirdPegasusConfig(PretrainedConfig):
             just in case (e.g., 1024 or 2048 or 4096).
         init_std (`float`, *optional*, defaults to 0.02):
             The standard deviation of the truncated_normal_initializer for initializing all weight matrices.
-        encoder_layerdrop: (`float`, *optional*, defaults to 0.0):
+        encoder_layerdrop (`float`, *optional*, defaults to 0.0):
             The LayerDrop probability for the encoder. See the [LayerDrop paper](see https://arxiv.org/abs/1909.11556)
             for more details.
-        decoder_layerdrop: (`float`, *optional*, defaults to 0.0):
+        decoder_layerdrop (`float`, *optional*, defaults to 0.0):
             The LayerDrop probability for the decoder. See the [LayerDrop paper](see https://arxiv.org/abs/1909.11556)
             for more details.
         use_cache (`bool`, *optional*, defaults to `True`):
